@@ -1,5 +1,7 @@
+/// This type represents all possible errors that can occur when generate or write JSON Schema string.
+/// 
 #[derive(Debug, thiserror::Error)]
-pub enum RschemaError {
+pub enum Error {
     #[error(transparent)]
     IoError(#[from] std::io::Error),
 
