@@ -4,10 +4,13 @@ use seq_macro::seq;
 use paste::paste;
 
 use crate::{
+    AdditionalProperties,
     ArrayProp,
     EnumProp,
     Items,
     NumericProp,
+    ObjectProp,
+    Properties,
     PropType,
     StringProp,
 };
@@ -28,6 +31,7 @@ use crate::{
 ///   - char
 ///   - str(&str)
 /// - **Compound types**:
+///   - [T; 0]
 ///   - tuples up to size 12
 /// - **Common standard library types**:
 ///   - String
@@ -45,11 +49,11 @@ pub trait Schematic {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType;
 
     fn __type_no_attr() -> PropType {
@@ -79,11 +83,11 @@ macro_rules! impl_for_str {
                 format: Option<String>,
                 minimum: Option<i64>,
                 maximum: Option<i64>,
-                multiple_of: Option<u64>,
-                exclusive_minimum: Option<bool>,
-                exclusive_maximum: Option<bool>,
-                min_items: Option<u64>,
-                max_items: Option<u64>,
+                multiple_of: Option<i64>,
+                exclusive_minimum: Option<i64>,
+                exclusive_maximum: Option<i64>,
+                min_items: Option<usize>,
+                max_items: Option<usize>,
             ) -> PropType {
                 PropType::String(StringProp {
                     min_length,
@@ -110,11 +114,11 @@ macro_rules! impl_for_num {
                 format: Option<String>,
                 minimum: Option<i64>,
                 maximum: Option<i64>,
-                multiple_of: Option<u64>,
-                exclusive_minimum: Option<bool>,
-                exclusive_maximum: Option<bool>,
-                min_items: Option<u64>,
-                max_items: Option<u64>,
+                multiple_of: Option<i64>,
+                exclusive_minimum: Option<i64>,
+                exclusive_maximum: Option<i64>,
+                min_items: Option<usize>,
+                max_items: Option<usize>,
             ) -> PropType {
                 PropType::Number(NumericProp {
                     minimum,
@@ -149,11 +153,11 @@ impl Schematic for char {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         PropType::String(StringProp {
             min_length: Some(1),
@@ -173,11 +177,11 @@ impl Schematic for bool {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         PropType::Boolean
     }
@@ -191,11 +195,11 @@ impl Schematic for () {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         PropType::Null
     }
@@ -213,11 +217,11 @@ macro_rules! impls {
                 format: Option<String>,
                 minimum: Option<i64>,
                 maximum: Option<i64>,
-                multiple_of: Option<u64>,
-                exclusive_minimum: Option<bool>,
-                exclusive_maximum: Option<bool>,
-                min_items: Option<u64>,
-                max_items: Option<u64>,
+                multiple_of: Option<i64>,
+                exclusive_minimum: Option<i64>,
+                exclusive_maximum: Option<i64>,
+                min_items: Option<usize>,
+                max_items: Option<usize>,
             ) -> PropType {
                 PropType::Array(ArrayProp {
                     items: Box::new(Items::Tuple(vec![
@@ -263,11 +267,11 @@ impl<T: Schematic> Schematic for Option<T> {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         PropType::Enum(EnumProp {
             any_of: vec![
@@ -278,6 +282,28 @@ impl<T: Schematic> Schematic for Option<T> {
     }
 }
 
+impl<T: Schematic, const N: usize> Schematic for [T; N] {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+    ) -> PropType {
+        PropType::Array(ArrayProp {
+            items: Box::new(Items::Single(T::__type_no_attr())),
+            min_items: Some(N),
+            max_items: Some(N),
+        })
+    }
+}
+
 impl<T: Schematic> Schematic for Vec<T> {
     fn __type(
         min_length: Option<u64>,
@@ -286,11 +312,11 @@ impl<T: Schematic> Schematic for Vec<T> {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         PropType::Array(ArrayProp {
             items: Box::new(Items::Single(T::__type_no_attr())),
@@ -308,11 +334,11 @@ impl<T: Schematic> Schematic for Box<T> {
         format: Option<String>,
         minimum: Option<i64>,
         maximum: Option<i64>,
-        multiple_of: Option<u64>,
-        exclusive_minimum: Option<bool>,
-        exclusive_maximum: Option<bool>,
-        min_items: Option<u64>,
-        max_items: Option<u64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
     ) -> PropType {
         T::__type(
             min_length,
@@ -329,3 +355,37 @@ impl<T: Schematic> Schematic for Box<T> {
         )
     }
 }
+
+// Collection types:
+// - BTreeMap<K, V>
+// - BTreeSet<T>
+// - BinaryHeap<T>
+// - HashMap<K, V, H>
+// - HashSet<T, H>
+// - LinkedList<T>
+// - VecDeque<T>
+// - Vec<T>
+
+impl<V: Schematic, S> Schematic for std::collections::HashMap<String, V, S> {
+    fn __type(
+        min_length: Option<u64>,
+        max_length: Option<u64>,
+        pattern: Option<String>,
+        format: Option<String>,
+        minimum: Option<i64>,
+        maximum: Option<i64>,
+        multiple_of: Option<i64>,
+        exclusive_minimum: Option<i64>,
+        exclusive_maximum: Option<i64>,
+        min_items: Option<usize>,
+        max_items: Option<usize>,
+    ) -> PropType {
+        PropType::Object(ObjectProp {
+            properties: Properties::new(),
+            required: vec![],
+            additional_properties: Box::new(
+                AdditionalProperties::Complex(V::__type_no_attr())
+            ),
+        })
+    }
+}
